// Generated on 2016-06-10 using
// generator-bootstrap4 0.0.2
'use strict';

// # Globbing
// for performance reasons we're only matching one level down:
// 'test/spec/{,*/}*.js'
// If you want to recursively match all subfolders, use:
// 'test/spec/**/*.js'

module.exports = function(grunt) {

  // Time how long tasks take. Can help when optimizing build times
  require('time-grunt')(grunt);

  // Automatically load required grunt tasks
  require('jit-grunt')(grunt);

  // Configurable paths
  var config = {
    app: 'source',
    dist: 'compiled',
    styleguide: 'styleguide'
  };

  // Define the configuration for all the tasks
  grunt.initConfig({

    // Project settings
    config: config,

    // Watches files for changes and runs tasks based on the changed files
    watch: {
      babel: {
        files: ['<%= config.app %>/scripts/{,*/}*.js'],
        tasks: ['babel:dist']
      },
      babelTest: {
        files: ['test/spec/{,*/}*.js'],
        tasks: ['babel:test', 'test:watch']
      },
      gruntfile: {
        files: ['Gruntfile.js']
      },
      sass: {
        files: ['<%= config.app %>/components/**/*.scss'],
        tasks: ['sass', 'postcss', 'kss']
      },
      twig: {
        files: ['<%= config.app %>/components/**/*.twig'],
        tasks: ['kss']
      },
      json: {
        files: ['<%= config.app %>/components/**/*.json'],
        tasks: ['kss']
      },
      kss_scripts: {
        files: ['<%= config.app %>/components/**/*.js'],
        tasks: ['kss']
      }
    },

    kss: {
      options: {
        css: ['kss-assets/kss.css'],
        homepage: 'homepage.md',
        title: 'Rijkshuisstijl Bootstrap Style Guide',
        builder: './<%= config.app %>/components/style-guide',
        'extend-drupal8': 'true'
      },
      dist: {
        src: ['<%= config.app %>'],
        dest: '<%= config.styleguide %>'
      }
    },

    browserSync: {
      options: {
        notify: false,
        background: true,
        watchOptions: {
          ignored: ''
        }
      },
      livereload: {
        options: {
          port: 9000,
          server: {
            baseDir: ['<%= config.styleguide %>'],
            routes: {
              '/compiled': './compiled'
            }
          }
        }
      },
      test: {
        options: {
          port: 9001,
          open: false,
          logLevel: 'silent',
          host: 'localhost',
          server: {
            baseDir: ['<%= config.styleguide %>', './test', config.app],
            routes: {
              '/bower_components': './bower_components'
            }
          }
        }
      },
      dist: {
        options: {
          background: false,
          server: '<%= config.styleguide %>'
        }
      }
    },

    // Empties folders to start fresh
    clean: {
      dist: {
        files: [{
          dot: true,
          src: [
            '<%= config.dist %>/*',
            '!<%= config.dist %>/.git*'
          ]
        }]
      },
      server: '<%= config.styleguide %>'
    },

    // Make sure code styles are up to par and there are no obvious mistakes
    eslint: {
      target: [
        'Gruntfile.js',
        '<%= config.app %>/scripts/{,*/}*.js',
        '!<%= config.app %>/scripts/vendor/*',
        'test/spec/{,*/}*.js'
      ]
    },

    // Mocha testing framework configuration options
    mocha: {
      all: {
        options: {
          run: true,
          urls: ['http://<%= browserSync.test.options.host %>:<%= browserSync.test.options.port %>/index.html']
        }
      }
    },

    // Compiles ES6 with Babel
    babel: {
      options: {
        sourceMap: true
      },
      dist: {
        files: [{
          expand: true,
          cwd: '<%= config.app %>/scripts',
          src: '{,*/}*.js',
          dest: '<%= config.dist %>/scripts',
          ext: '.js'
        }]
      },
      test: {
        files: [{
          expand: true,
          cwd: 'test/spec',
          src: '{,*/}*.js',
          dest: '<%= config.dist %>/spec',
          ext: '.js'
        }]
      }
    },

    // Compiles Sass to CSS and generates necessary files if requested
    sass: {
      dist: { // Target
        files: [{
          expand: true,
          cwd: '<%= config.app %>/components/schemes',
          src: ['*.scss'],
          dest: '<%= config.dist %>/styles',
          ext: '.css'
        }]
      },
      styleguide: {
        options: { // Target options
          style: 'expanded'
        },
        files: {
          //'<%= config.dist %>/styles/main.css': '<%= config.app %>/components/main.scss',
          '<%= config.app %>/components/style-guide/kss-assets/kss.css': '<%= config.app %>/components/style-guide/kss-assets/kss.scss',
        }
      }
    },

    postcss: {
      options: {
        map: true,
        processors: [
          // Add vendor prefixed styles
          require('autoprefixer')({
            browsers: [
              'Android 2.3',
              'Android >= 4',
              'Chrome >= 35',
              'Firefox >= 31',
              // Note: Edge versions in Autoprefixer & Can I Use refer to the EdgeHTML rendering engine version,
              // NOT the Edge app version shown in Edge's "About" screen.
              // For example, at the time of writing, Edge 20 on an up-to-date system uses EdgeHTML 12.
              // See also https://github.com/Fyrd/caniuse/issues/1928
              'Edge >= 12',
              'Explorer >= 9',
              'iOS >= 7',
              'Opera >= 12',
              'Safari >= 7.1'
            ]
          })
        ]
      },
      dist: {
        files: [{
          expand: true,
          cwd: '<%= config.dist %>/styles/',
          src: '{,*/}*.css',
          dest: '<%= config.dist %>/styles/'
        }]
      }
    },

    // The following *-min tasks produce minified files in the dist folder
    imagemin: {
      dist: {
        files: [{
          expand: true,
          cwd: '<%= config.app %>/images',
          src: '{,*/}*.{gif,jpeg,jpg,png}',
          dest: '<%= config.dist %>/images'
        }]
      }
    },

    svgmin: {
      dist: {
        files: [{
          expand: true,
          cwd: '<%= config.app %>/images',
          src: '{,*/}*.svg',
          dest: '<%= config.dist %>/images'
        }]
      }
    },

    uglify: {
      dist: {
        files: {
          '<%= config.dist %>/scripts/main.js': [
            '<%= config.dist %>/scripts/main.js'
          ]
        }
      }
    },

    // Concat Javascript files.
    concat: {
      options: {
        separator: ';',
      },
      dist: {
        files: {
          '<%= config.app %>/components/style-guide/kss-assets/bower.js': ['bower_components/jquery/dist/jquery.js', 'bower_components/tether/dist/js/tether.js', 'bower_components/bootstrap/dist/js/bootstrap.js', 'bower_components/slick-carousel/slick/slick.js', 'bower_components/jquery-focuspoint/js/jquery.focuspoint.js'],
          '<%= config.dist %>/scripts/bootstrap.js': ['bower_components/tether/dist/js/tether.js', 'bower_components/bootstrap/dist/js/bootstrap.js'],
          '<%= config.dist %>/scripts/slick.js': ['bower_components/slick-carousel/slick/slick.js'],
          '<%= config.dist %>/scripts/focuspoint.js': ['bower_components/jquery-focuspoint/js/jquery.focuspoint.js'],        },
      },
    },

    // Copies remaining files to places other tasks can use
    copy: {
      dist: {
        files: [{
          expand: true,
          dot: true,
          cwd: '<%= config.app %>',
          dest: '<%= config.dist %>',
          src: [
            '*.{ico,png,txt}',
            'images/{,*/}*.*',
            'fonts/{,*/}*.*'
          ]
        }]
      }
    },

    // Run some tasks in parallel to speed up build process
    concurrent: {
      server: [
        'babel:dist',
        'sass'
      ],
      test: [
        'babel'
      ],
      dist: [
        'babel',
        'sass',
        'imagemin',
        'svgmin'
      ]
    },

    // use auto install to do a bower update/install
    auto_install: {
      local: {
        options: {
          npm: false
        }
      }
    }

  });

  grunt.registerTask('styleguide', [
    'kss'
  ]);

  grunt.registerTask('serve', 'start the server and preview your app', function(target) {
    if (target === 'dist') {
      return grunt.task.run(['build', 'browserSync:dist']);
    }

    grunt.task.run([
      'auto_install',
      'clean:server',
      'concurrent:server',
      'postcss',
      'kss',
      'copy:dist',
      'concat',
      'uglify',
      'browserSync:livereload',
      'watch'
    ]);
  });

  grunt.registerTask('server', function(target) {
    grunt.log.warn('The `server` task has been deprecated. Use `grunt serve` to start a server.');
    grunt.task.run([target ? ('serve:' + target) : 'serve']);
  });

  grunt.registerTask('test', function(target) {
    if (target !== 'watch') {
      grunt.task.run([
        'clean:server',
        'concurrent:test',
        'postcss'
      ]);
    }

    grunt.task.run([
      'browserSync:test',
      'mocha'
    ]);
  });

  grunt.registerTask('build', [
    'auto_install',
    'clean:dist',
    'concurrent:dist',
    'postcss',
    'concat',
    'uglify',
    'copy:dist',
    'kss'
  ]);

  grunt.registerTask('default', [
    'newer:eslint',
    'test',
    'build'
  ]);

};
